<?php
/**
 * Here the REST API endpoint of the plugin are registered.
 *
 * @package real-voice-pro
 */

/**
 * This class should be used to work with the REST API endpoints of the plugin.
 */
class Daextrevop_Rest {

	/**
	 * The singleton instance of the class.
	 *
	 * @var null
	 */
	protected static $instance = null;

	/**
	 * An instance of the shared class.
	 *
	 * @var Daextrevop_Shared|null
	 */
	private $shared = null;

	/**
	 * An instance of the audio files management class.
	 *
	 * @var Daextrevop_Audio_Files_Management|null
	 */
	private $audio_files_management = null;

	/**
	 * An instance of the class used to write the custom CSS file.
	 *
	 * @var Daextrevop_Write_Css_File|null
	 */
	private $write_css_file = null;

	/**
	 * Constructor.
	 */
	private function __construct() {

		// Assign an instance of the shared class.
		$this->shared = Daextrevop_Shared::get_instance();

		// Audio Files Management.
		$this->audio_files_management = Daextrevop_Audio_Files_Management::get_instance();

		// Write CSS File.
		$this->write_css_file = Daextrevop_Write_Css_File::get_instance();

		/**
		 * Add custom routes to the Rest API.
		 */
		add_action( 'rest_api_init', array( $this, 'rest_api_register_route' ) );
	}

	/**
	 * Create a singleton instance of the class.
	 *
	 * @return self|null
	 */
	public static function get_instance() {

		if ( null === self::$instance ) {
			self::$instance = new self();
		}

		return self::$instance;
	}

	/**
	 * Add custom routes to the Rest API.
	 *
	 * @return void
	 */
	public function rest_api_register_route() {

		// Add the POST 'daext-real-voice/v1/create-audio-file/' endpoint to the Rest API.
		register_rest_route(
			'daext-real-voice/v1',
			'/create-audio-file/',
			array(
				'methods'             => 'POST',
				'callback'            => array( $this, 'rest_api_daext_real_voice_create_audio_file_callback' ),
				'permission_callback' => '__return_true',
			)
		);

		// Add the POST 'real-voice/v1/delete-audio-file/' endpoint to the Rest API.
		register_rest_route(
			'daext-real-voice/v1',
			'/delete-audio-file/',
			array(
				'methods'             => 'POST',
				'callback'            => array( $this, 'rest_api_daext_real_voice_delete_audio_file_callback' ),
				'permission_callback' => '__return_true',
			)
		);

		// Add the GET 'real-voice-pro/v1/options/' endpoint to the Rest API.
		register_rest_route(
			'real-voice-pro/v1',
			'/read-options/',
			array(
				'methods'             => 'POST',
				'callback'            => array( $this, 'rest_api_real_voice_pro_read_options_callback' ),
				'permission_callback' => '__return_true',
			)
		);

		// Add the POST 'real-voice-pro/v1/options/' endpoint to the Rest API.
		register_rest_route(
			'real-voice-pro/v1',
			'/options',
			array(
				'methods'             => 'POST',
				'callback'            => array( $this, 'rest_api_real_voice_pro_update_options_callback' ),
				'permission_callback' => '__return_true',
			)
		);

		// Add the GET 'real-voice-pro/v1/requests/' endpoint to the Rest API.
		register_rest_route(
			'real-voice-pro/v1',
			'/requests/',
			array(
				'methods'             => 'POST',
				'callback'            => array( $this, 'rest_api_real_voice_pro_read_requests_callback' ),
				'permission_callback' => '__return_true',
			)
		);
	}

	/**
	 * Callback for the POST 'daext-real-voice/v1/audio/' endpoint of the Rest API.
	 *
	 *  This method is in the following contexts:
	 *
	 *  - To create a new audio file in the post editor.
	 *
	 * @param object $request The REST API request parameters.
	 *
	 * @return WP_Error|WP_REST_Response
	 */
	public function rest_api_daext_real_voice_create_audio_file_callback( $request ) {

		if ( ! current_user_can( 'manage_options' ) ) {
			return new WP_Error(
				'rest_update_error',
				'Sorry, you are not allowed to generate a new audio file.',
				array( 'status' => 403 )
			);
		}

		// Check if the nonce is valid.
		$post_id = intval( $request->get_param( 'id' ), 10 );
		$nonce   = $request->get_param( 'nonce' );
		if ( ! wp_verify_nonce( $nonce, 'daextrevop_create_audio_file_nonce' ) ) {
			return new WP_Error( 'invalid_nonce', 'Invalid nonce.', array( 'status' => 403 ) );
		}

		$text_to_speech_converter = get_option( $this->shared->get( 'slug' ) . '_text_to_speech_converter' );

		$result = $this->audio_files_management->generate_audio_file( $post_id, $text_to_speech_converter );

		// Generate the response.
		$response = new WP_REST_Response( $result );

		return $response;
	}

	/**
	 * Callback for the POST 'daext-real-voice/v1/delete-audio-file/' endpoint of the Rest API.
	 *
	 *   This method is in the following contexts:
	 *
	 *   - To delete an audio file when the "Delete File" button available in the "Audio File" post sidebar is clicked.
	 *
	 * @param object $request Object with the request data.
	 *
	 * @return WP_Error|WP_REST_Response
	 */
	public function rest_api_daext_real_voice_delete_audio_file_callback( $request ) {

		if ( ! current_user_can( 'manage_options' ) ) {
			return new WP_Error(
				'rest_update_error',
				'Sorry, you are not allowed to delete an new audio file.',
				array( 'status' => 403 )
			);
		}

		// Check if the nonce is valid.
		$nonce = $request->get_param( 'nonce' );
		if ( ! wp_verify_nonce( $nonce, 'daextrevop_delete_audio_file_nonce' ) ) {
			return new WP_Error( 'invalid_nonce', 'Invalid nonce.', array( 'status' => 403 ) );
		}

		// Get and sanitize data --------------------------------------------------------------------------------------.
		$post_id = intval( $request->get_param( 'id' ), 10 );

		// Delete the audio file.
		$result = $this->audio_files_management->delete_audio_file( $post_id );

		// Generate the response.
		$response = new WP_REST_Response( $result );

		return $response;
	}

	/**
	 * Callback for the GET 'real-voice-pro/v1/options' endpoint of the Rest API.
	 *
	 *   This method is in the following contexts:
	 *
	 *  - To retrieve the plugin options in the "Options" menu.
	 *
	 * @param object $request The data of the REST API request.
	 *
	 * @return WP_Error|WP_REST_Response
	 */
	public function rest_api_real_voice_pro_read_options_callback( $request ) {

		// Check the capability.
		if ( ! current_user_can( 'manage_options' ) ) {
			return new WP_Error(
				'rest_read_error',
				'Sorry, you are not allowed to view the Real Voice options.',
				array( 'status' => 403 )
			);
		}

		// Check if the nonce is valid.
		$nonce = $request->get_param( 'read_options_nonce' );
		if ( ! wp_verify_nonce( $nonce, 'daextrevop_read_options_nonce' ) ) {
			return new WP_Error( 'invalid_nonce', 'Invalid nonce.', array( 'status' => 403 ) );
		}

		// Generate the response.
		$response = array();
		foreach ( $this->shared->get( 'options' ) as $key => $value ) {
			$response[ $key ] = get_option( $key );
		}

		// Prepare the response.
		$response = new WP_REST_Response( $response );

		return $response;
	}

	/**
	 * Callback for the POST 'real-voice-pro/v1/options' endpoint of the Rest API.
	 *
	 * This method is in the following contexts:
	 *
	 *  - To update the plugin options in the "Options" menu.
	 *
	 * @param object $request The request data.
	 *
	 * @return WP_REST_Response
	 */
	public function rest_api_real_voice_pro_update_options_callback( $request ) {

		if ( ! current_user_can( 'manage_options' ) ) {
			return new WP_Error(
				'rest_update_error',
				'Sorry, you are not allowed to update the Real Voice options.',
				array( 'status' => 403 )
			);
		}

		// Check if the nonce is valid.
		$nonce = $request->get_param( 'update_options_nonce' );
		if ( ! wp_verify_nonce( $nonce, 'daextrevop_update_options_nonce' ) ) {
			return new WP_Error( 'invalid_nonce', 'Invalid nonce.', array( 'status' => 403 ) );
		}

		// get and sanitize data --------------------------------------------------------------------------------------.

		// TTS --------------------------------------------------------------------------------------------------------.

		// General.
		$daextrevop_text_to_speech_converter = $request->get_param( 'daextrevop_text_to_speech_converter' ) !== null ? sanitize_key( $request->get_param( 'daextrevop_text_to_speech_converter' ) ) : null;

		// SpeechSynthesis (Web Speech API).
		$daextrevop_speech_synthesis_lang   = $request->get_param( 'daextrevop_speech_synthesis_lang' ) !== null ? sanitize_text_field( $request->get_param( 'daextrevop_speech_synthesis_lang' ) ) : null;
		$daextrevop_speech_synthesis_pitch  = $request->get_param( 'daextrevop_speech_synthesis_pitch' ) !== null ? floatval( $request->get_param( 'daextrevop_speech_synthesis_pitch' ) ) : null;
		$daextrevop_speech_synthesis_rate   = $request->get_param( 'daextrevop_speech_synthesis_rate' ) !== null ? floatval( $request->get_param( 'daextrevop_speech_synthesis_rate' ) ) : null;
		$daextrevop_speech_synthesis_volume = $request->get_param( 'daextrevop_speech_synthesis_volume' ) !== null ? floatval( $request->get_param( 'daextrevop_speech_synthesis_volume' ) ) : null;

		// Elevenlabs.
		$daextrevop_elevenlabs_api_key                    = $request->get_param( 'daextrevop_elevenlabs_api_key' ) !== null ? sanitize_text_field( $request->get_param( 'daextrevop_elevenlabs_api_key' ) ) : null;
		$daextrevop_elevenlabs_voice_id                   = $request->get_param( 'daextrevop_elevenlabs_voice_id' ) !== null ? sanitize_text_field( $request->get_param( 'daextrevop_elevenlabs_voice_id' ) ) : null;
		$daextrevop_elevenlabs_custom_voice_id                   = $request->get_param( 'daextrevop_elevenlabs_custom_voice_id' ) !== null ? sanitize_text_field( $request->get_param( 'daextrevop_elevenlabs_custom_voice_id' ) ) : null;
		$daextrevop_elevenlabs_optimize_streaming_latency = $request->get_param( 'daextrevop_elevenlabs_optimize_streaming_latency' ) !== null ? intval( $request->get_param( 'daextrevop_elevenlabs_optimize_streaming_latency' ), 10 ) : null;
		$daextrevop_elevenlabs_stability                  = $request->get_param( 'daextrevop_elevenlabs_stability' ) !== null ? floatval( $request->get_param( 'daextrevop_elevenlabs_stability' ) ) : null;
		$daextrevop_elevenlabs_similarity_boost           = $request->get_param( 'daextrevop_elevenlabs_similarity_boost' ) !== null ? floatval( $request->get_param( 'daextrevop_elevenlabs_similarity_boost' ) ) : null;
		$daextrevop_elevenlabs_style                      = $request->get_param( 'daextrevop_elevenlabs_style' ) !== null ? sanitize_text_field( $request->get_param( 'daextrevop_elevenlabs_style' ) ) : null;
		$daextrevop_elevenlabs_use_speaker_boost          = $request->get_param( 'daextrevop_elevenlabs_use_speaker_boost' ) !== null ? intval( $request->get_param( 'daextrevop_elevenlabs_use_speaker_boost' ), 10 ) : null;
		$daextrevop_elevenlabs_model_id                   = $request->get_param( 'daextrevop_elevenlabs_model_id' ) !== null ? sanitize_key( $request->get_param( 'daextrevop_elevenlabs_model_id' ) ) : null;

		// Amazon Polly.
		$daextrevop_aws_access_key_id     = $request->get_param( 'daextrevop_aws_access_key_id' ) !== null ? sanitize_text_field( $request->get_param( 'daextrevop_aws_access_key_id' ) ) : null;
		$daextrevop_aws_secret_access_key = $request->get_param( 'daextrevop_aws_secret_access_key' ) !== null ? sanitize_text_field( $request->get_param( 'daextrevop_aws_secret_access_key' ) ) : null;
		$daextrevop_aws_region            = $request->get_param( 'daextrevop_aws_region' ) !== null ? sanitize_key( $request->get_param( 'daextrevop_aws_region' ) ) : null;
		$daextrevop_aws_voice_id          = $request->get_param( 'daextrevop_aws_voice_id' ) !== null ? sanitize_text_field( $request->get_param( 'daextrevop_aws_voice_id' ) ) : null;
		$daextrevop_aws_engine            = $request->get_param( 'daextrevop_aws_engine' ) !== null ? sanitize_key( $request->get_param( 'daextrevop_aws_engine' ) ) : null;
		$daextrevop_aws_language_code     = $request->get_param( 'daextrevop_aws_language_code' ) !== null ? sanitize_text_field( $request->get_param( 'daextrevop_aws_language_code' ) ) : null;
		$daextrevop_aws_lexicon_names     = $request->get_param( 'daextrevop_aws_lexicon_names' ) !== null ? sanitize_text_field( $request->get_param( 'daextrevop_aws_lexicon_names' ) ) : null;
		$daextrevop_aws_output_format     = $request->get_param( 'daextrevop_aws_output_format' ) !== null ? sanitize_key( $request->get_param( 'daextrevop_aws_output_format' ) ) : null;
		$daextrevop_aws_sample_rate       = $request->get_param( 'daextrevop_aws_sample_rate' ) !== null ? sanitize_text_field( $request->get_param( 'daextrevop_aws_sample_rate' ) ) : null;

		// Google Cloud Text-to-Speech AI.
		$daextrevop_google_cloud_text_to_speech_api_key          = $request->get_param( 'daextrevop_google_cloud_text_to_speech_api_key' ) !== null ? sanitize_text_field( $request->get_param( 'daextrevop_google_cloud_text_to_speech_api_key' ) ) : null;
		$daextrevop_google_cloud_audio_config_audio_encoding     = $request->get_param( 'daextrevop_google_cloud_audio_config_audio_encoding' ) !== null ? sanitize_text_field( $request->get_param( 'daextrevop_google_cloud_audio_config_audio_encoding' ) ) : null;
		$daextrevop_google_cloud_audio_config_speaking_rate      = $request->get_param( 'daextrevop_google_cloud_audio_config_speaking_rate' ) !== null ? floatval( $request->get_param( 'daextrevop_google_cloud_audio_config_speaking_rate' ) ) : null;
		$daextrevop_google_cloud_audio_config_pitch              = $request->get_param( 'daextrevop_google_cloud_audio_config_pitch' ) !== null ? floatval( $request->get_param( 'daextrevop_google_cloud_audio_config_pitch' ) ) : null;
		$daextrevop_google_cloud_audio_config_volume_gain_db     = $request->get_param( 'daextrevop_google_cloud_audio_config_volume_gain_db' ) !== null ? floatval( $request->get_param( 'daextrevop_google_cloud_audio_config_volume_gain_db' ) ) : null;
		$daextrevop_google_cloud_audio_config_sample_rate_hertz  = $request->get_param( 'daextrevop_google_cloud_audio_config_sample_rate_hertz' ) !== null ? intval( $request->get_param( 'daextrevop_google_cloud_audio_config_sample_rate_hertz' ), 10 ) : null;
		$daextrevop_google_cloud_audio_config_effects_profile_id = $request->get_param( 'daextrevop_google_cloud_audio_config_effects_profile_id' ) !== null && is_array($request->get_param( 'daextrevop_google_cloud_audio_config_effects_profile_id' ))  ? array_map( 'sanitize_key', $request->get_param( 'daextrevop_google_cloud_audio_config_effects_profile_id' ) ) : null;
		$daextrevop_google_cloud_voice_language_code             = $request->get_param( 'daextrevop_google_cloud_voice_language_code' ) !== null ? sanitize_text_field( $request->get_param( 'daextrevop_google_cloud_voice_language_code' ) ) : null;
		$daextrevop_google_cloud_voice_name                      = $request->get_param( 'daextrevop_google_cloud_voice_name' ) !== null ? sanitize_text_field( $request->get_param( 'daextrevop_google_cloud_voice_name' ) ) : null;

		// Azure Text-to-Speech.
		$daextrevop_azure_speech_resource_key       = $request->get_param( 'daextrevop_azure_speech_resource_key' ) !== null ? sanitize_text_field( $request->get_param( 'daextrevop_azure_speech_resource_key' ) ) : null;
		$daextrevop_azure_region                    = $request->get_param( 'daextrevop_azure_region' ) !== null ? sanitize_key( $request->get_param( 'daextrevop_azure_region' ) ) : null;
		$daextrevop_azure_user_agent                = $request->get_param( 'daextrevop_azure_user_agent' ) !== null ? sanitize_text_field( $request->get_param( 'daextrevop_azure_user_agent' ) ) : null;
		$daextrevop_azure_x_microsoft_output_format = $request->get_param( 'daextrevop_azure_x_microsoft_output_format' ) !== null ? sanitize_text_field( $request->get_param( 'daextrevop_azure_x_microsoft_output_format' ) ) : null;
		$daextrevop_azure_voice_short_name          = $request->get_param( 'daextrevop_azure_voice_short_name' ) !== null ? sanitize_text_field( $request->get_param( 'daextrevop_azure_voice_short_name' ) ) : null;

		// General ----------------------------------------------------------------------------------------------------.
		$daextrevop_post_types         = $request->get_param( 'daextrevop_post_types' ) !== null ? array_map( 'sanitize_text_field', $request->get_param( 'daextrevop_post_types' ) ) : null;
		$daextrevop_text_before        = $request->get_param( 'daextrevop_text_before' ) !== null ? sanitize_text_field( $request->get_param( 'daextrevop_text_before' ) ) : null;
		$daextrevop_text_after         = $request->get_param( 'daextrevop_text_after' ) !== null ? sanitize_text_field( $request->get_param( 'daextrevop_text_after' ) ) : null;
		$daextrevop_download_button    = $request->get_param( 'daextrevop_download_button' ) !== null ? sanitize_text_field( $request->get_param( 'daextrevop_download_button' ) ) : null;
		$daextrevop_speech_text_before = $request->get_param( 'daextrevop_speech_text_before' ) !== null ? sanitize_text_field( $request->get_param( 'daextrevop_speech_text_before' ) ) : null;
		$daextrevop_speech_text_after  = $request->get_param( 'daextrevop_speech_text_after' ) !== null ? sanitize_text_field( $request->get_param( 'daextrevop_speech_text_after' ) ) : null;
		$daextrevop_read_title         = $request->get_param( 'daextrevop_read_title' ) !== null ? intval( $request->get_param( 'daextrevop_read_title' ), 10 ) : null;

		// Style ------------------------------------------------------------------------------------------------------.
		$daextrevop_custom_player_background_color   = $request->get_param( 'daextrevop_custom_player_background_color' ) !== null ? sanitize_text_field( $request->get_param( 'daextrevop_custom_player_background_color' ) ) : null;
		$daextrevop_custom_player_border_color       = $request->get_param( 'daextrevop_custom_player_border_color' ) !== null ? sanitize_text_field( $request->get_param( 'daextrevop_custom_player_border_color' ) ) : null;
		$daextrevop_custom_player_icons_color        = $request->get_param( 'daextrevop_custom_player_icons_color' ) !== null ? sanitize_text_field( $request->get_param( 'daextrevop_custom_player_icons_color' ) ) : null;
		$daextrevop_custom_player_slider_thumb_color = $request->get_param( 'daextrevop_custom_player_slider_thumb_color' ) !== null ? sanitize_text_field( $request->get_param( 'daextrevop_custom_player_slider_thumb_color' ) ) : null;
		$daextrevop_custom_player_slider_track_color = $request->get_param( 'daextrevop_custom_player_slider_track_color' ) !== null ? sanitize_text_field( $request->get_param( 'daextrevop_custom_player_slider_track_color' ) ) : null;
		$daextrevop_custom_player_text_color         = $request->get_param( 'daextrevop_custom_player_text_color' ) !== null ? sanitize_text_field( $request->get_param( 'daextrevop_custom_player_text_color' ) ) : null;
		$daextrevop_custom_player_font_family        = $request->get_param( 'daextrevop_custom_player_font_family' ) !== null ? sanitize_text_field( $request->get_param( 'daextrevop_custom_player_font_family' ) ) : null;
		$daextrevop_custom_player_font_size          = $request->get_param( 'daextrevop_custom_player_font_size' ) !== null ? intval( $request->get_param( 'daextrevop_custom_player_font_size' ), 10 ) : null;
		$daextrevop_custom_player_font_style         = $request->get_param( 'daextrevop_custom_player_font_style' ) !== null ? sanitize_key( $request->get_param( 'daextrevop_custom_player_font_style' ) ) : null;
		$daextrevop_custom_player_font_weight        = $request->get_param( 'daextrevop_custom_player_font_weight' ) !== null ? intval( $request->get_param( 'daextrevop_custom_player_font_weight' ), 10 ) : null;
		$daextrevop_custom_player_line_height        = $request->get_param( 'daextrevop_custom_player_line_height' ) !== null ? intval( $request->get_param( 'daextrevop_custom_player_line_height' ), 10 ) : null;
		$daextrevop_custom_player_drop_shadow        = $request->get_param( 'daextrevop_custom_player_drop_shadow' ) !== null ? intval( $request->get_param( 'daextrevop_custom_player_drop_shadow' ), 10 ) : null;
		$daextrevop_custom_player_drop_shadow_color  = $request->get_param( 'daextrevop_custom_player_drop_shadow_color' ) !== null ? sanitize_text_field( $request->get_param( 'daextrevop_custom_player_drop_shadow_color' ) ) : null;

		// Advanced ---------------------------------------------------------------------------------------------------.
		$daextrevop_generate_audio_on_post_view          = $request->get_param( 'daextrevop_generate_audio_on_post_view' ) !== null ? intval( $request->get_param( 'daextrevop_generate_audio_on_post_view' ), 10 ) : null;
		$daextrevop_responsive_breakpoint                = $request->get_param( 'daextrevop_responsive_breakpoint' ) !== null ? intval( $request->get_param( 'daextrevop_responsive_breakpoint' ), 10 ) : null;
		$daextrevop_responsive_breakpoint_2              = $request->get_param( 'daextrevop_responsive_breakpoint_2' ) !== null ? intval( $request->get_param( 'daextrevop_responsive_breakpoint_2' ), 10 ) : null;
		$daextrevop_google_font_url                      = $request->get_param( 'daextrevop_google_font_url' ) !== null ? esc_url_raw( $request->get_param( 'daextrevop_google_font_url' ) ) : null;
		$daextrevop_development_mode                     = $request->get_param( 'daextrevop_development_mode' ) !== null ? intval( $request->get_param( 'daextrevop_development_mode' ), 10 ) : null;
		$daextrevop_statistics_menu_capability           = $request->get_param( 'daextrevop_statistics_menu_capability' ) !== null ? sanitize_key( $request->get_param( 'daextrevop_statistics_menu_capability' ) ) : null;
		$daextrevop_editor_tools_capability              = $request->get_param( 'daextrevop_editor_tools_capability' ) !== null ? sanitize_key( $request->get_param( 'daextrevop_editor_tools_capability' ) ) : null;
		$daextrevop_post_types_ui         = $request->get_param( 'daextrevop_post_types_ui' ) !== null ? array_map( 'sanitize_text_field', $request->get_param( 'daextrevop_post_types_ui' ) ) : null;

		// Validation -------------------------------------------------------------------------------------------------.

		if ( null === $daextrevop_text_to_speech_converter ) {
			$daextrevop_text_to_speech_converter = get_option( 'daextrevop_text_to_speech_converter' );
		}

		if ( null === $daextrevop_speech_synthesis_lang ) {
			$daextrevop_speech_synthesis_lang = get_option( 'daextrevop_speech_synthesis_lang' );
		}

		if ( null === $daextrevop_speech_synthesis_pitch ) {
			$daextrevop_speech_synthesis_pitch = get_option( 'daextrevop_speech_synthesis_pitch' );
		}

		if ( null === $daextrevop_speech_synthesis_rate ) {
			$daextrevop_speech_synthesis_rate = get_option( 'daextrevop_speech_synthesis_rate' );
		}

		if ( null === $daextrevop_speech_synthesis_volume ) {
			$daextrevop_speech_synthesis_volume = get_option( 'daextrevop_speech_synthesis_volume' );
		}

		if ( null === $daextrevop_post_types ) {
			$daextrevop_post_types = get_option( 'daextrevop_post_types' );
		}

		if ( null === $daextrevop_text_before ) {
			$daextrevop_text_before = get_option( 'daextrevop_text_before' );
		}

		if ( null === $daextrevop_text_after ) {
			$daextrevop_text_after = get_option( 'daextrevop_text_after' );
		}

		if ( null === $daextrevop_download_button ) {
			$daextrevop_download_button = get_option( 'daextrevop_download_button' );
		}

		if ( null === $daextrevop_speech_text_before ) {
			$daextrevop_speech_text_before = get_option( 'daextrevop_speech_text_before' );
		}

		if ( null === $daextrevop_speech_text_after ) {
			$daextrevop_speech_text_after = get_option( 'daextrevop_speech_text_after' );
		}

		if ( null === $daextrevop_elevenlabs_api_key ) {
			$daextrevop_elevenlabs_api_key = get_option( 'daextrevop_elevenlabs_api_key' );
		}

		if ( null === $daextrevop_elevenlabs_voice_id ) {
			$daextrevop_elevenlabs_voice_id = get_option( 'daextrevop_elevenlabs_voice_id' );
		}

		if ( null === $daextrevop_elevenlabs_custom_voice_id ) {
			$daextrevop_elevenlabs_custom_voice_id = get_option( 'daextrevop_elevenlabs_custom_voice_id' );
		}

		if ( null === $daextrevop_elevenlabs_optimize_streaming_latency ) {
			$daextrevop_elevenlabs_optimize_streaming_latency = get_option( 'daextrevop_elevenlabs_optimize_streaming_latency' );
		}

		if ( null === $daextrevop_elevenlabs_stability ) {
			$daextrevop_elevenlabs_stability = get_option( 'daextrevop_elevenlabs_stability' );
		}

		if ( null === $daextrevop_elevenlabs_similarity_boost ) {
			$daextrevop_elevenlabs_similarity_boost = get_option( 'daextrevop_elevenlabs_similarity_boost' );
		}

		if ( null === $daextrevop_elevenlabs_style ) {
			$daextrevop_elevenlabs_style = get_option( 'daextrevop_elevenlabs_style' );
		}

		if ( null === $daextrevop_elevenlabs_use_speaker_boost ) {
			$daextrevop_elevenlabs_use_speaker_boost = get_option( 'daextrevop_elevenlabs_use_speaker_boost' );
		}

		if ( null === $daextrevop_elevenlabs_model_id ) {
			$daextrevop_elevenlabs_model_id = get_option( 'daextrevop_elevenlabs_model_id' );
		}

		if ( null === $daextrevop_aws_access_key_id ) {
			$daextrevop_aws_access_key_id = get_option( 'daextrevop_aws_access_key_id' );
		}

		if ( null === $daextrevop_aws_secret_access_key ) {
			$daextrevop_aws_secret_access_key = get_option( 'daextrevop_aws_secret_access_key' );
		}

		if ( null === $daextrevop_aws_region ) {
			$daextrevop_aws_region = get_option( 'daextrevop_aws_region' );
		}

		if ( null === $daextrevop_aws_voice_id ) {
			$daextrevop_aws_voice_id = get_option( 'daextrevop_aws_voice_id' );
		}

		if ( null === $daextrevop_aws_engine ) {
			$daextrevop_aws_engine = get_option( 'daextrevop_aws_engine' );
		}

		if ( null === $daextrevop_aws_language_code ) {
			$daextrevop_aws_language_code = get_option( 'daextrevop_aws_language_code' );
		}

		if ( null === $daextrevop_aws_lexicon_names ) {
			$daextrevop_aws_lexicon_names = get_option( 'daextrevop_aws_lexicon_names' );
		}

		if ( null === $daextrevop_aws_output_format ) {
			$daextrevop_aws_output_format = get_option( 'daextrevop_aws_output_format' );
		}

		if ( null === $daextrevop_aws_sample_rate ) {
			$daextrevop_aws_sample_rate = get_option( 'daextrevop_aws_sample_rate' );
		}

		if ( null === $daextrevop_google_cloud_text_to_speech_api_key ) {
			$daextrevop_google_cloud_text_to_speech_api_key = get_option( 'daextrevop_google_cloud_text_to_speech_api_key' );
		}

		if ( null === $daextrevop_google_cloud_audio_config_audio_encoding ) {
			$daextrevop_google_cloud_audio_config_audio_encoding = get_option( 'daextrevop_google_cloud_audio_config_audio_encoding' );
		}

		if ( null === $daextrevop_google_cloud_audio_config_speaking_rate ) {
			$daextrevop_google_cloud_audio_config_speaking_rate = get_option( 'daextrevop_google_cloud_audio_config_speaking_rate' );
		}

		if ( null === $daextrevop_google_cloud_audio_config_pitch ) {
			$daextrevop_google_cloud_audio_config_pitch = get_option( 'daextrevop_google_cloud_audio_config_pitch' );
		}

		if ( null === $daextrevop_google_cloud_audio_config_volume_gain_db ) {
			$daextrevop_google_cloud_audio_config_volume_gain_db = get_option( 'daextrevop_google_cloud_audio_config_volume_gain_db' );
		}

		if ( null === $daextrevop_google_cloud_audio_config_sample_rate_hertz ) {
			$daextrevop_google_cloud_audio_config_sample_rate_hertz = get_option( 'daextrevop_google_cloud_audio_config_sample_rate_hertz' );
		}

		if ( null === $daextrevop_google_cloud_audio_config_effects_profile_id ) {
			$daextrevop_google_cloud_audio_config_effects_profile_id = get_option( 'daextrevop_google_cloud_audio_config_effects_profile_id' );
		}

		if ( null === $daextrevop_google_cloud_voice_language_code ) {
			$daextrevop_google_cloud_voice_language_code = get_option( 'daextrevop_google_cloud_voice_language_code' );
		}

		if ( null === $daextrevop_google_cloud_voice_name ) {
			$daextrevop_google_cloud_voice_name = get_option( 'daextrevop_google_cloud_voice_name' );
		}

		if ( null === $daextrevop_azure_speech_resource_key ) {
			$daextrevop_azure_speech_resource_key = get_option( 'daextrevop_azure_speech_resource_key' );
		}

		if ( null === $daextrevop_azure_region ) {
			$daextrevop_azure_region = get_option( 'daextrevop_azure_region' );
		}

		if ( null === $daextrevop_azure_user_agent ) {
			$daextrevop_azure_user_agent = get_option( 'daextrevop_azure_user_agent' );
		}

		if ( null === $daextrevop_azure_x_microsoft_output_format ) {
			$daextrevop_azure_x_microsoft_output_format = get_option( 'daextrevop_azure_x_microsoft_output_format' );
		}

		if ( null === $daextrevop_azure_voice_short_name ) {
			$daextrevop_azure_voice_short_name = get_option( 'daextrevop_azure_voice_short_name' );
		}

		if ( null === $daextrevop_read_title ) {
			$daextrevop_read_title = get_option( 'daextrevop_read_title' );
		}

		if ( null === $daextrevop_custom_player_background_color ) {
			$daextrevop_custom_player_background_color = get_option( 'daextrevop_custom_player_background_color' );
		}

		if ( null === $daextrevop_custom_player_border_color ) {
			$daextrevop_custom_player_border_color = get_option( 'daextrevop_custom_player_border_color' );
		}

		if ( null === $daextrevop_custom_player_icons_color ) {
			$daextrevop_custom_player_icons_color = get_option( 'daextrevop_custom_player_icons_color' );
		}

		if ( null === $daextrevop_custom_player_slider_thumb_color ) {
			$daextrevop_custom_player_slider_thumb_color = get_option( 'daextrevop_custom_player_slider_thumb_color' );
		}

		if ( null === $daextrevop_custom_player_slider_track_color ) {
			$daextrevop_custom_player_slider_track_color = get_option( 'daextrevop_custom_player_slider_track_color' );
		}

		if ( null === $daextrevop_custom_player_text_color ) {
			$daextrevop_custom_player_text_color = get_option( 'daextrevop_custom_player_text_color' );
		}

		if ( null === $daextrevop_custom_player_font_family ) {
			$daextrevop_custom_player_font_family = get_option( 'daextrevop_custom_player_font_family' );
		}

		if ( null === $daextrevop_custom_player_font_size ) {
			$daextrevop_custom_player_font_size = get_option( 'daextrevop_custom_player_font_size' );
		}

		if ( null === $daextrevop_custom_player_font_style ) {
			$daextrevop_custom_player_font_style = get_option( 'daextrevop_custom_player_font_style' );
		}

		if ( null === $daextrevop_custom_player_font_weight ) {
			$daextrevop_custom_player_font_weight = get_option( 'daextrevop_custom_player_font_weight' );
		}

		if ( null === $daextrevop_custom_player_line_height ) {
			$daextrevop_custom_player_line_height = get_option( 'daextrevop_custom_player_line_height' );
		}

		if ( null === $daextrevop_custom_player_drop_shadow ) {
			$daextrevop_custom_player_drop_shadow = get_option( 'daextrevop_custom_player_drop_shadow' );
		}

		if ( null === $daextrevop_custom_player_drop_shadow_color ) {
			$daextrevop_custom_player_drop_shadow_color = get_option( 'daextrevop_custom_player_drop_shadow_color' );
		}

		if ( null === $daextrevop_generate_audio_on_post_view ) {
			$daextrevop_generate_audio_on_post_view = get_option( 'daextrevop_generate_audio_on_post_view' );
		}

		if ( null === $daextrevop_responsive_breakpoint ) {
			$daextrevop_responsive_breakpoint = get_option( 'daextrevop_responsive_breakpoint' );
		}

		if ( null === $daextrevop_responsive_breakpoint_2 ) {
			$daextrevop_responsive_breakpoint_2 = get_option( 'daextrevop_responsive_breakpoint_2' );
		}

		if ( null === $daextrevop_development_mode ) {
			$daextrevop_development_mode = get_option( 'daextrevop_development_mode' );
		}

		if ( null === $daextrevop_statistics_menu_capability ) {
			$daextrevop_statistics_menu_capability = get_option( 'daextrevop_statistics_menu_capability' );
		}

		if ( null === $daextrevop_editor_tools_capability ) {
			$daextrevop_editor_tools_capability = get_option( 'daextrevop_editor_tools_capability' );
		}

		if ( null === $daextrevop_post_types_ui ) {
			$daextrevop_post_types_ui = get_option( 'daextrevop_post_types_ui' );
		}

		// Update the options -----------------------------------------------------------------------------------------.

		// TTS --------------------------------------------------------------------------------------------------------.
		update_option( 'daextrevop_text_to_speech_converter', $daextrevop_text_to_speech_converter );
		update_option( 'daextrevop_speech_synthesis_lang', $daextrevop_speech_synthesis_lang );
		update_option( 'daextrevop_speech_synthesis_pitch', $daextrevop_speech_synthesis_pitch );
		update_option( 'daextrevop_speech_synthesis_rate', $daextrevop_speech_synthesis_rate );
		update_option( 'daextrevop_speech_synthesis_volume', $daextrevop_speech_synthesis_volume );
		update_option( 'daextrevop_elevenlabs_api_key', $daextrevop_elevenlabs_api_key );
		update_option( 'daextrevop_elevenlabs_voice_id', $daextrevop_elevenlabs_voice_id );
		update_option( 'daextrevop_elevenlabs_custom_voice_id', $daextrevop_elevenlabs_custom_voice_id );
		update_option( 'daextrevop_elevenlabs_optimize_streaming_latency', $daextrevop_elevenlabs_optimize_streaming_latency );
		update_option( 'daextrevop_elevenlabs_stability', $daextrevop_elevenlabs_stability );
		update_option( 'daextrevop_elevenlabs_similarity_boost', $daextrevop_elevenlabs_similarity_boost );
		update_option( 'daextrevop_elevenlabs_style', $daextrevop_elevenlabs_style );
		update_option( 'daextrevop_elevenlabs_use_speaker_boost', $daextrevop_elevenlabs_use_speaker_boost );
		update_option( 'daextrevop_elevenlabs_model_id', $daextrevop_elevenlabs_model_id );
		update_option( 'daextrevop_aws_access_key_id', $daextrevop_aws_access_key_id );
		update_option( 'daextrevop_aws_secret_access_key', $daextrevop_aws_secret_access_key );
		update_option( 'daextrevop_aws_region', $daextrevop_aws_region );
		update_option( 'daextrevop_aws_voice_id', $daextrevop_aws_voice_id );
		update_option( 'daextrevop_aws_engine', $daextrevop_aws_engine );
		update_option( 'daextrevop_aws_language_code', $daextrevop_aws_language_code );
		update_option( 'daextrevop_aws_lexicon_names', $daextrevop_aws_lexicon_names );
		update_option( 'daextrevop_aws_output_format', $daextrevop_aws_output_format );
		update_option( 'daextrevop_aws_sample_rate', $daextrevop_aws_sample_rate );
		update_option( 'daextrevop_google_cloud_text_to_speech_api_key', $daextrevop_google_cloud_text_to_speech_api_key );
		update_option( 'daextrevop_google_cloud_audio_config_audio_encoding', $daextrevop_google_cloud_audio_config_audio_encoding );
		update_option( 'daextrevop_google_cloud_audio_config_speaking_rate', $daextrevop_google_cloud_audio_config_speaking_rate );
		update_option( 'daextrevop_google_cloud_audio_config_pitch', $daextrevop_google_cloud_audio_config_pitch );
		update_option( 'daextrevop_google_cloud_audio_config_volume_gain_db', $daextrevop_google_cloud_audio_config_volume_gain_db );
		update_option( 'daextrevop_google_cloud_audio_config_sample_rate_hertz', $daextrevop_google_cloud_audio_config_sample_rate_hertz );
		update_option( 'daextrevop_google_cloud_audio_config_effects_profile_id', $daextrevop_google_cloud_audio_config_effects_profile_id );
		update_option( 'daextrevop_google_cloud_voice_language_code', $daextrevop_google_cloud_voice_language_code );
		update_option( 'daextrevop_google_cloud_voice_name', $daextrevop_google_cloud_voice_name );
		update_option( 'daextrevop_azure_speech_resource_key', $daextrevop_azure_speech_resource_key );
		update_option( 'daextrevop_azure_region', $daextrevop_azure_region );
		update_option( 'daextrevop_azure_user_agent', $daextrevop_azure_user_agent );
		update_option( 'daextrevop_azure_x_microsoft_output_format', $daextrevop_azure_x_microsoft_output_format );
		update_option( 'daextrevop_azure_voice_short_name', $daextrevop_azure_voice_short_name );

		// General.
		update_option( 'daextrevop_post_types', $daextrevop_post_types );
		update_option( 'daextrevop_text_before', $daextrevop_text_before );
		update_option( 'daextrevop_text_after', $daextrevop_text_after );
		update_option( 'daextrevop_download_button', $daextrevop_download_button );
		update_option( 'daextrevop_speech_text_before', $daextrevop_speech_text_before );
		update_option( 'daextrevop_speech_text_after', $daextrevop_speech_text_after );
		update_option( 'daextrevop_read_title', $daextrevop_read_title );

		// Style ------------------------------------------------------------------------------------------------------.
		update_option( 'daextrevop_custom_player_background_color', $daextrevop_custom_player_background_color );
		update_option( 'daextrevop_custom_player_border_color', $daextrevop_custom_player_border_color );
		update_option( 'daextrevop_custom_player_icons_color', $daextrevop_custom_player_icons_color );
		update_option( 'daextrevop_custom_player_slider_thumb_color', $daextrevop_custom_player_slider_thumb_color );
		update_option( 'daextrevop_custom_player_slider_track_color', $daextrevop_custom_player_slider_track_color );
		update_option( 'daextrevop_custom_player_text_color', $daextrevop_custom_player_text_color );
		update_option( 'daextrevop_custom_player_font_family', $daextrevop_custom_player_font_family );
		update_option( 'daextrevop_custom_player_font_size', $daextrevop_custom_player_font_size );
		update_option( 'daextrevop_custom_player_font_style', $daextrevop_custom_player_font_style );
		update_option( 'daextrevop_custom_player_font_weight', $daextrevop_custom_player_font_weight );
		update_option( 'daextrevop_custom_player_line_height', $daextrevop_custom_player_line_height );
		update_option( 'daextrevop_custom_player_drop_shadow', $daextrevop_custom_player_drop_shadow );
		update_option( 'daextrevop_custom_player_drop_shadow_color', $daextrevop_custom_player_drop_shadow_color );

		// Advanced ---------------------------------------------------------------------------------------------------.
		update_option( 'daextrevop_generate_audio_on_post_view', $daextrevop_generate_audio_on_post_view );
		update_option( 'daextrevop_responsive_breakpoint', $daextrevop_responsive_breakpoint );
		update_option( 'daextrevop_responsive_breakpoint_2', $daextrevop_responsive_breakpoint_2 );
		update_option( 'daextrevop_google_font_url', $daextrevop_google_font_url );
		update_option( 'daextrevop_development_mode', $daextrevop_development_mode );
		update_option( 'daextrevop_statistics_menu_capability', $daextrevop_statistics_menu_capability );
		update_option( 'daextrevop_editor_tools_capability', $daextrevop_editor_tools_capability );
		update_option( 'daextrevop_post_types_ui', $daextrevop_post_types_ui );

		/*
		 * Write the custom-[blog_id].css file or die if the file can't be created or modified.
		 */
		if ( $this->write_css_file->write_custom_css() === false ) {
			die( "The plugin can't write files in the upload directory." );
		}

		return new WP_REST_Response( 'Data successfully added.', '200' );
	}

	/**Callback for the GET 'real-voice-pro/v1/requests' endpoint of the Rest API.
	 *
	 * This method is in the following contexts:
	 *
	 * - In the "Statistics" menu to retrieve the data of the HTTP requests performed to the Text to Speech API
	 * services.
	 *
	 * @param object $request The request data.
	 *
	 * @return WP_Error|WP_REST_Response
	 */
	public function rest_api_real_voice_pro_read_requests_callback( $request ) {

		// Check the capability.
		if ( ! current_user_can( 'manage_options' ) ) {
			return new WP_Error(
				'rest_read_error',
				'Sorry, you are not allowed to view the Real Voice requests.',
				array( 'status' => 403 )
			);
		}

		// Check if the nonce is valid.
		$nonce = $request->get_param( 'nonce' );
		if ( ! wp_verify_nonce( $nonce, 'daextrevop_read_requests_nonce' ) ) {
			return new WP_Error( 'invalid_nonce', 'Invalid nonce.', array( 'status' => 403 ) );
		}

		$date_range = intval( $request->get_param( 'date_range' ), 10 );
		$converter  = intval( $request->get_param( 'converter' ), 10 );

		// Create the query for the date range ------------------------------------------------------------------------.

		switch ( $date_range ) {

			// 7 Days.
			case 0:
				// 7 - 1
				$days = 6;
				break;

			// 28 Days.
			case 1:
				// 28 - 1.
				$days = 27;
				break;

			// 3 Months.

			case 2:
				// 90 - 1.
				$days = 89;
				break;

			// 6 Months

			case 3:
				// 180 - 1
				$days = 179;
				break;

			// 12 Months.

			case 4:
				// 365 - 1.
				$days = 364;
				break;

			// 16 Months.

			case 5:
				// 486 - 1.
				$days = 485;
				break;

		}

		$start_date = gmdate( 'Y-m-d', strtotime( gmdate( 'Y-m-d' ) . ' - ' . $days . ' days' ) );
		$end_date   = gmdate( 'Y-m-d', strtotime( gmdate( 'Y-m-d' ) . ' - 0 days' ) );

		global $wpdb;
		$request_query_part = $wpdb->prepare(
			'WHERE (request_date BETWEEN %s AND %s)',
			$start_date,
			$end_date
		);

		// Create the query for the converter -------------------------------------------------------.
		$converter_query_part = '';
		switch ( $converter ) {

			// All.
			case 0:
				$converter_query_part = '';
				break;

			// Amazon Polly.
			case 1:
				$converter_query_part = "AND converter = 'amazon-polly'";
				break;

			// Google Text-to-Speech API.
			case 2:
				$converter_query_part = "AND converter = 'google-text-to-speech-ai'";
				break;

			// Azure Text to Speech.
			case 3:
				$converter_query_part = "AND converter = 'azure-text-to-speech'";
				break;

			// ElevenLabs.
			case 4:
				$converter_query_part = "AND converter = 'elevenlabs'";
				break;

		}

		global $wpdb;
		$table_name = $wpdb->prefix . $this->shared->get( 'slug' ) . '_request';

		// Use get_results() to create a query where the results between $start_date and $end_date are returned.
		$requests = $wpdb->get_results(
			"
			SELECT *
			FROM $table_name
			$request_query_part
			$converter_query_part
			order by request_id DESC
			"
		);

		/**
		 * Create a new array with the cumulative value of the requests per day. These data will be used to generate
		 * the line chart.
		 */
		$requests_per_day = array();
		for ( $i = 0; $i <= $days; $i++ ) {
			$today                      = gmdate( 'Y-m-d', strtotime( gmdate( 'Y-m-d' ) . " - $i days" ) );
			$requests_per_day[ $today ] = 0;
			foreach ( $requests as $request ) {
				if ( $request->request_date === $today ) {
					$requests_per_day[ $today ] = $requests_per_day[ $today ] + 1;
				}
			}
		}

		// Calculate $total_requests ------------------------------------------------------------.
		$total_requests = count( $requests );

		// Calculate $total_characters ----------------------------------------------------------.
		$total_characters = 0;
		foreach ( $requests as $request ) {
			$total_characters += $request->characters;
		}

		// Calculate the average amount of requests per day - $average_requests_per_day.
		$average_requests_per_day = round( $total_requests / 7 );

		// Prepare the date for table and store them in the $prepared_requests array -----------------.
		$prepared_requests = array();
		foreach ( $requests as $key => $request ) {
			$prepared_requests[] = array(
				'request_id'    => $request->request_id,
				'converter'     => $this->get_service_name_from_slug( $request->converter ),
				'request_date'  => wp_date( get_option( 'date_format' ), strtotime( $request->request_date ) ),
				'characters'    => $request->characters,
				'error'         => $request->error,
				'error_message' => $request->error_message,
			);
		}

		/**
		 * Prepare the response with the data for the various sections of the
		 * dashboard.
		 */
		$response = array(
			'summary'  => array(
				'total_requests'   => $total_requests,
				'total_characters' => $total_characters,
				'requests_per_day' => $average_requests_per_day,
			),
			'chart'    => $requests_per_day,
			'requests' => $prepared_requests,
		);

		// Prepare the response.
		$response = new WP_REST_Response( $response );

		return $response;
	}

	/**
	 * Get the text-to-speech service name from the slug.
	 *
	 * @param string $service_slug The slug of the text-to-speech service.
	 * @return string The full name of the text-to-speech service.
	 */
	private function get_service_name_from_slug( $service_slug ) {

		switch ( $service_slug ) {

			case 'amazon-polly':
				$service_name = 'Amazon Polly';
				break;

			case 'google-text-to-speech-ai':
				$service_name = 'Google Text-to-Speech AI';
				break;

			case 'azure-text-to-speech':
				$service_name = 'Azure Text-to-Speech';
				break;

			case 'elevenlabs':
				$service_name = 'ElevenLabs';
				break;

		}

		return $service_name;
	}

}
